class InteractionData:
    """Marker class for all interaction data events.
    
    !!! tip "Children"
        [`ApplicationCommandDataModel`][scurrypy.api.interactions.ApplicationCommandDataModel]
        [`MessageComponentDataModel`][scurrypy.api.interactions.MessageComponentDataModel]
        [`ModalDataModel`][scurrypy.api.interactions.ModalDataModel]
    """
    __slots__ = ()
